<?php

use uqpay\payment\config\ConfigOfAPI;
use uqpay\payment\config\security\SecurityUqpayException;
use uqpay\payment\Constants;
use uqpay\payment\Gateway;
use uqpay\payment\model\OrderRefund;
use uqpay\payment\model\PaymentResult;
use uqpay\payment\model\PostRedirectData;
use uqpay\payment\UqpayException;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

// phpcs:disable WordPress.Files.FileName

abstract class WC_UQPAY_Payment_Gateway extends WC_Payment_Gateway_CC {

	/**
	 * @var Gateway
	 */
	protected $uqpay_API;
	protected $test_mode;

	public function init_uqpay_api() {
		$main_settings   = get_option( 'woocommerce_uqpay_union_online_settings' );
		$this->test_mode = 'yes' === $main_settings['test_mode'];
		$merchant_id     = $main_settings['merchant_id'];
		$prv_key         = $this->test_mode ? $main_settings['test_private_key'] : $main_settings['private_key'];
		$prv_key_type    = $this->test_mode ? $main_settings['test_private_key_type'] : $main_settings['private_key_type'];
		$pub_key         = $this->test_mode ? $main_settings['test_publishable_key'] : $main_settings['publishable_key'];

		$api_config      = ConfigOfAPI::builder(
			$prv_key,
			$prv_key_type,
			$pub_key,
			$merchant_id,
			$this->test_mode
		);
		$this->uqpay_API = new Gateway( $api_config );
		$this->uqpay_API->setHttpClient( WC_UQPAY_API::get_instance() );
	}

	public function card_scheme_icons() {
		return array(
			WC_UQPAY_Constants::CARD_SCHEME_AMEX       => '<img src="' . WC_UQPAY_PLUGIN_URL . '/assets/images/amex.svg" class="uqpay-scheme-icon uqpay-icon" alt="American Express" />',
			WC_UQPAY_Constants::CARD_SCHEME_VISA       => '<img src="' . WC_UQPAY_PLUGIN_URL . '/assets/images/visa.svg" class="uqpay-scheme-icon uqpay-icon" alt="Visa" />',
			WC_UQPAY_Constants::CARD_SCHEME_DISCOVER   => '<img src="' . WC_UQPAY_PLUGIN_URL . '/assets/images/discover.svg" class="uqpay-scheme-icon uqpay-icon" alt="Discover" />',
			WC_UQPAY_Constants::CARD_SCHEME_JCB        => '<img src="' . WC_UQPAY_PLUGIN_URL . '/assets/images/jcb.svg" class="uqpay-scheme-icon uqpay-icon" alt="JCB" />',
			WC_UQPAY_Constants::CARD_SCHEME_MASTERCARD => '<img src="' . WC_UQPAY_PLUGIN_URL . '/assets/images/mastercard.svg" class="uqpay-scheme-icon uqpay-icon" alt="Mastercard" />',
			WC_UQPAY_Constants::CARD_SCHEME_UNION      => '<img src="' . WC_UQPAY_PLUGIN_URL . '/assets/images/union.svg" class="uqpay-scheme-icon uqpay-icon" alt="Union Pay" />',
		);
	}

	public function payment_icons() {
		return array(
			WC_UQPAY_Constants::PAYMENT_NAME_UNION => '<img src="' . WC_UQPAY_PLUGIN_URL . '/assets/images/union.svg" class="uqpay-payment-icon uqpay-icon" alt="UnionPay" />',
		);
	}

	/**
	 * Check if we need to make gateways available.
	 *
	 * @since 1.0.0
	 */
	public function is_available() {
		if ( 'yes' === $this->enabled && ! empty( $this->uqpay_API ) && $this->uqpay_API->isAvailable() ) {
			return true;
		}

		return false;
	}

	/**
	 * Gets the Payment Order URL linked to UQPAY Merchant dashboard
	 *
	 * @param WC_Order $order
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 * @version 1.0.0
	 */
	public function get_transaction_url( $order ) {
		if ( $this->test_mode ) {
			$this->view_transaction_url = 'https://merchant.uqpay.net/#/transactions/order/detail/%s';
		} else {
			$this->view_transaction_url = 'https://merchant.uqpay.com/#/transactions/order/detail/%s';
		}

		return parent::get_transaction_url( $order );
	}


	/**
	 * Gets the return URL from redirects
	 *
	 * @param null $order
	 * @param null $id
	 * @param PostRedirectData $client_post
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 * @version 1.0.0
	 */
	public function get_uqpay_return_url( $order = null, $id = null, PostRedirectData $client_post = null ) {
		if ( is_object( $order ) ) {
			if ( empty( $id ) ) {
				$id = uniqid();
			}

			$order_id = WC_UQPAY_Helper::is_wc_lt( '3.0' ) ? $order->id : $order->get_id();
			$args     = array(
				'order_id' => $order_id,
			);
			if ( empty( $client_post ) ) {
				$args['utm_nooverride'] = '1';
				$args['uqpay_result']   = '1';
			} else {
				$args['client_post_url']  = $client_post->url;
				$args['client_post_body'] = base64_encode( json_encode( $client_post->body ) );
			}


			return esc_url_raw( add_query_arg( $args, $this->get_return_url( $order ) ) );
		}

		return esc_url_raw( add_query_arg( array( 'utm_nooverride' => '1' ), $this->get_return_url() ) );
	}

	/**
	 * generate the request body for the payment
	 *
	 * @param $order
	 *
	 * @since 1.0.0
	 * @version 1.0.0
	 */
	public function generate_payment_request( $order ) {
		$settings  = get_option( 'woocommerce_uqpay_settings', array() );
		$post_data = array();

		//TODO 待完善
	}

	/**
	 * @param WC_Order $order
	 *
	 * @throws UqpayException
	 */
	public function validate_minimum_order_amount( WC_Order $order ) {
		if ( $order->get_total() * 100 < 1 ) {
			/* translators: 1) dollar amount */
			throw new UqpayException( 'Did not meet minimum amount', sprintf( __( 'Sorry, the minimum allowed order total is %1$s to use this payment method.', 'uqpay-payment-gateway' ), wc_price( 1 / 100 ) ) );
		}
	}

	/**
	 * refund an order
	 *
	 * @param int $order_id
	 * @param null $amount
	 * @param string $reason
	 *
	 * @return bool
	 *
	 * @since 1.0.0
	 * @version 1.0.0
	 */
	public function process_refund( $order_id, $amount = null, $reason = '' ) {
		$order = wc_get_order( $order_id );
		if ( ! $order ) {
			return false;
		}

		$refunded = WC_UQPAY_Helper::is_wc_lt( '3.0' )
			? get_post_meta( $order_id, '_uqpay_refund_id', true )
			: $order->get_meta( '_uqpay_refund_id', true );

		if ( ! empty( $refunded ) ) {
			WC_UQPAY_Logger::log( 'The order ID: ' . $order_id . 'already has an refund authorized' );

			return false;
		}

		$orderRefund                 = new OrderRefund();
		$orderRefund->uqpay_order_id = WC_UQPAY_Helper::is_wc_lt( '3.0' ) ? get_post_meta( $order->id, '_transaction_id', true ) : $order->get_transaction_id();
		$orderRefund->order_id       = time() . '_refund';
		$orderRefund->amount         = empty( $amount ) ? $order->get_total() : $amount;
		$orderRefund->callback_url   = WC_UQPAY_Helper::get_async_callback_url();
		$orderRefund->date           = time();
		$orderRefund->extend_info    = array(
			'origin_order_id' => WC_UQPAY_Helper::is_wc_lt( '3.0' ) ? $order->id : $order->get_id(),
			'reason'          => $reason
		);

		if ( empty( $this->uqpay_API ) || ! $this->uqpay_API->isAvailable() ) {
			$this->init_uqpay_api();
		}
		try {
			$refund_result = $this->uqpay_API->refund( $orderRefund );
			if ( Constants::ORDER_STATE_SUCCESS == $refund_result->state
			     || Constants::ORDER_STATE_SYNC_SUCCESS == $refund_result->state
			     || Constants::ORDER_STATE_REFUNDING ) {
				WC_UQPAY_Helper::is_wc_lt( '3.0' ) ? update_post_meta( $order_id, '_uqpay_refund_id', $refund_result->uqpay_order_id ) : $order->update_meta_data( '_uqpay_refund_id', $refund_result->uqpay_order_id );

				/* translators: 1) dollar amount 2) transaction id 3) refund message */
				$refund_message = sprintf( __( 'Refund authorized (Refunded %1$s - UQPAY Refund ID: %2$s - Reason: %3$s), The async notification from UQPAY will final confirm this refund.', 'uqpay-payment-gateway' ), $amount, $refund_result->uqpay_order_id, $reason );

				$order->add_order_note( $refund_message );
				WC_UQPAY_Logger::log( 'Success: ' . html_entity_decode( wp_strip_all_tags( $refund_message ) ) );
				if ( is_callable( array( $order, 'save' ) ) ) {
					$order->save();
				}

				return true;
			}
			WC_Stripe_Logger::log( 'Error: ' . $refund_result->code . ' ' . $refund_result->message );

			return false;
		} catch ( ReflectionException $e ) {
			WC_UQPAY_Logger::log( 'UQPAY Refund error: ' . $e->getMessage() );
		} catch ( SecurityUqpayException $e ) {
			WC_UQPAY_Logger::log( 'UQPAY Refund error: ' . $e->getMessage() );
		} catch ( UqpayException $e ) {
			WC_UQPAY_Logger::log( 'UQPAY Refund error: ' . $e->getMessage() . print_r( $orderRefund ) );
		}

		return false;
	}

	/**
	 * @param PaymentResult $payment_result
	 * @param WC_Order $order
	 * @param bool $sync
	 *
	 * @return PaymentResult
	 * @throws UqpayException
	 * @throws WC_Data_Exception
	 */
	public function process_payment_result( PaymentResult $payment_result, $order, $sync = true ) {
		WC_UQPAY_Logger::log( 'Processing response: ' . print_r( $payment_result, true ) );
		$order_id = WC_UQPAY_Helper::is_wc_lt( '3.0' ) ? $order->id : $order->get_id();

		if ( $sync ) {
			if ( $payment_result->state == Constants::ORDER_STATE_FAILED || $payment_result->state == Constants::ORDER_STATE_SYNC_FAILED ) {
				/* translators: error code */
				throw new UqpayException( 'UQPAY payment failed: ' . $payment_result->message, sprintf( __( 'Payment error code: %s.', 'uqpay-payment-gateway' ), $payment_result->code ) );
			}
			if ( $payment_result->state == Constants::ORDER_STATE_SUCCESS
			     || $payment_result->state == Constants::ORDER_STATE_SYNC_SUCCESS
			) {
				WC_UQPAY_Helper::is_wc_lt( '3.0' ) ? update_post_meta( $order_id, '_transaction_id', $payment_result->uqpay_order_id ) : $order->set_transaction_id( $payment_result->uqpay_order_id );

				if ( $order->has_status( array( 'pending', 'failed' ) ) ) {
					WC_UQPAY_Helper::is_wc_lt( '3.0' ) ? $order->reduce_order_stock() : wc_reduce_stock_levels( $order_id );
				}

				/* translators: uqpay order id */
				$order->update_status( 'on-hold', sprintf( __( 'UQPAY payment authorized (Payment ID: %s). The async notification from UQPAY will final confirm this trade.', 'uqpay-payment-gateway' ), $payment_result->uqpay_order_id ) );
			}
		} else {
			// Store charge data.
			WC_UQPAY_Helper::is_wc_lt( '3.0' ) ? update_post_meta( $order_id, '_uqpay_trans_confirmed_date', $payment_result->date ) : $order->update_meta_data( '_uqpay_trans_confirmed_date', $payment_result->date );
			if ( Constants::ORDER_STATE_FAILED == $payment_result->state ) {
				$localized_message = sprintf( __( 'Payment error code: %s.', 'uqpay-payment-gateway' ), $payment_result->code );
				$order->update_status( 'failed', $localized_message );
				throw new UqpayException( print_r( $payment_result, true ), $localized_message );
			}

			if ( Constants::ORDER_STATE_SUCCESS == $payment_result->state ) {
				$order->payment_complete( $payment_result->uqpay_order_id );
				/* translators: uqpay order id */
				$message = sprintf( __( 'UQPAY payment complete (UQPAY Order ID: %s)', 'uqpay-payment-gateway' ), $payment_result->uqpay_order_id );
				$order->add_order_note( $message );
			}
		}

		if ( is_callable( array( $order, 'save' ) ) ) {
			$order->save();
		}

		return $payment_result;
	}

	/**
	 * Sends the failed order email to admin.
	 *
	 * @param int $order_id
	 *
	 * @return null
	 * @since 1.0.0
	 * @version 1.0.0
	 */
	public function send_failed_order_email( $order_id ) {
		$emails = WC()->mailer()->get_emails();
		if ( ! empty( $emails ) && ! empty( $order_id ) ) {
			$emails['WC_Email_Failed_Order']->trigger( $order_id );
		}
	}
}