<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use uqpay\payment\model\PaymentOrder;
use uqpay\payment\PayMethodHelper;
use uqpay\payment\UqpayException;

/**
 * Class WC_Gateway_UQPAY_UNION_Online
 *
 * @extends WC_Payment_Gateway
 *
 * @since 1.0.0
 * @version 1.0.0
 */
class WC_Gateway_UQPAY_UNION_Online extends WC_UQPAY_Payment_Gateway {
	private $method_id = PayMethodHelper::UNION_SECURE_PAY;

	/**
	 * WC_Gateway_UQPAY_UNION_Online constructor.
	 */
	public function __construct() {
		$this->id                 = 'uqpay_union_online';
		$this->method_title       = __( 'UQPAY UnionPay Online', 'uqpay-payment-gateway' );
		/* translators: 1) merchant register url 2) merchant dashboard url */
		$this->method_description = sprintf( __( 'UQPAY Payment. <a href="%1$s" target="_blank">Sign up</a> for a UQPAY merchant, and <a href="%2$s" target="_blank">get your UQPAY account keys</a>.', 'uqpay-payment-gateway' ), 'https://merchant.uqpay.com/#/register', 'https://merchant.uqpay.cn' );
		$this->supports           = array(
			'products',
			'refunds',
		);
		$this->init_uqpay_api();
		$this->init_form_fields();

		$this->init_settings();

		// Get setting values.
		$this->title                = $this->get_option( 'title' );
		$this->description          = $this->get_option( 'description' );
		$this->enabled              = $this->get_option( 'enabled' );

		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'payment_scripts' ) );
	}

	public function init_form_fields() {
		$this->form_fields = require( WC_UQPAY_PLUGIN_PATH . '/includes/admin/uqpay-union-online-settings.php' );
	}

	/**
	 * Checks to see if all criteria is met before showing payment method.
	 * @return bool
	 * @since 1.0.0
	 * @version 1.0.0
	 */
	public function is_available() {
		return parent::is_available();
	}

	/**
	 * Get_icon function.
	 *
	 * @since 1.0.0
	 * @version 4.0.0
	 * @return string
	 */
	public function get_icon() {
		$icons = $this->payment_icons();

		$icons_str = '';

		$icons_str .= isset( $icons[WC_UQPAY_Constants::PAYMENT_NAME_UNION] ) ? $icons[WC_UQPAY_Constants::PAYMENT_NAME_UNION] : '';

		return apply_filters( 'woocommerce_gateway_icon', $icons_str, $this->id );
	}

	public function payment_fields() {
		echo wpautop( wp_kses_post( $this->get_description() ) );
	}

	public function payment_scripts() {
		wp_register_style( 'uqpay_payment_styles', plugins_url( 'assets/css/uqpay-payment-styles.css', WC_UQPAY_MAIN_FILE ), array(), WC_UQPAY_VERSION );
		wp_enqueue_style( 'uqpay_payment_styles' );
	}

	public function process_payment( $order_id ) {
		$order = wc_get_order( $order_id );
		try {
			// This will throw exception if not valid.
			$this->validate_minimum_order_amount($order);

			$payment_order = new PaymentOrder();
			$payment_order->method_id = $this->method_id;
			$payment_order->order_id = WC_UQPAY_Helper::is_wc_lt( '3.0' ) ? $order->id : $order->get_id();
			/* translators: 1) website name 2) order number */
			$payment_order->trans_name = sprintf( __( '%1$s - Order %2$s', 'uqpay-payment-gateway' ), wp_specialchars_decode( get_bloginfo( 'name' ), ENT_QUOTES ), $order->get_order_number() );
			$payment_order->amount = $order->get_total();
			$payment_order->currency = WC_UQPAY_Helper::is_wc_lt( '3.0' ) ? $order->get_order_currency() : $order->get_currency();
			$payment_order->date = time();
			$payment_order->client_ip = $order->get_customer_ip_address();
			$payment_order->quantity = $order->get_item_count();
			$payment_order->return_url = $this->get_uqpay_return_url($order);
			$payment_order->callback_url = WC_UQPAY_Helper::get_async_callback_url();

			$process_request = $this->uqpay_API->pay($payment_order);
			WC_UQPAY_Logger::log( 'Info: Redirecting to UnionPay Online...' );
			return array(
				'result' => 'success',
				'redirect' => $this->get_uqpay_return_url($order, null, $process_request->redirect),
			);
		} catch (UqpayException $ex) {
			$message = empty($ex->getLocalizedMessage()) ? $ex->getMessage() : $ex->getLocalizedMessage();
			wc_add_notice( $message, 'error' );
			WC_UQPAY_Logger::log( 'Error: ' . $ex->getMessage() );
			$statuses = array( 'pending', 'failed' );

			if ( $order->has_status( $statuses ) ) {
				$this->send_failed_order_email( $order_id );
			}

			return array(
				'result'   => 'fail',
				'redirect' => '',
			);
		}
	}
}